//
//  AppDelegate.m
//  Layer Rotate
//
//  Created by Matt Long on 10/25/08.
//

#import "AppDelegate.h"

CGFloat DegreesToRadians(CGFloat degrees)
{
    return degrees * M_PI / 180;
}

NSNumber* DegreesToNumber(CGFloat degrees)
{
    return [NSNumber numberWithFloat:
            DegreesToRadians(degrees)];
}

@implementation AppDelegate

- (void)awakeFromNib;
{
    [view setWantsLayer:YES];
    
    CGColorRef black = CGColorCreateGenericRGB(0.0, 0.0, 0.0, 1.0);
    CGColorRef orange = CGColorCreateGenericRGB(1.0, 0.56, 0.0, 1.0);
    
    [[view layer] setBackgroundColor:black];
    
    needleLayer = [CALayer layer];
    [needleLayer setFrame:CGRectMake(0.0, 0.0, [view frame].size.width, [view frame].size.height)];
    
    [needleLayer setCornerRadius:[needleLayer bounds].size.width / 2];
    [needleLayer setMasksToBounds:YES];
    [needleLayer setBackgroundColor:orange];
    [needleLayer setDelegate:self];
    
    [needleLayer addAnimation:[self rotateAnimation] forKey:@"rotate"];

    CFRelease(orange);
    CFRelease(black);

    degreesTextLayer = [CATextLayer layer];
    [degreesTextLayer setString:@"0.000000"];
    CGRect frame = [needleLayer bounds];
    frame.size.height = frame.size.height/2;
    [degreesTextLayer setFrame:frame];
    [degreesTextLayer setAlignmentMode:kCAAlignmentCenter];

    [needleLayer addSublayer:degreesTextLayer];
    
    [needleLayer setNeedsDisplay];
    
    [[view layer] addSublayer:needleLayer];
}

- (IBAction)sliderChange:(id)sender;
{
    [degreesTextLayer setString:[NSString stringWithFormat:@"%f", [slider floatValue]]];
    [needleLayer addAnimation:[self rotateAnimation] forKey:@"rotate"];
    [needleLayer setNeedsDisplay];
}

- (CAAnimation*)rotateAnimation;
{
	CABasicAnimation * animation;
	animation = [CABasicAnimation 
                 animationWithKeyPath:@"transform.rotation.z"];
    
    [animation setFromValue:DegreesToNumber(previousValue)];
    [animation setToValue:DegreesToNumber([slider floatValue])];
    
    [animation setRemovedOnCompletion:NO];
    [animation setFillMode:kCAFillModeForwards];
    
    previousValue = [slider floatValue];
    
	return animation;
}

- (void)drawLayer:(CALayer *)layer inContext:(CGContextRef)context;
{
    if( layer == needleLayer )
    {
        CGMutablePathRef path = CGPathCreateMutable();
        
        CGPathMoveToPoint(path,NULL, [view bounds].size.width/2.0, 0.0);
        CGPathAddLineToPoint(path, NULL, [view bounds].size.width/2.0, [view bounds].size.height);

        CGPathMoveToPoint(path,NULL, 0.0, [view bounds].size.height/2.0);
        CGPathAddLineToPoint(path, NULL, [view bounds].size.width, [view bounds].size.height/2.0);
        
        CGColorRef black =
        CGColorCreateGenericRGB(0.0, 0.0, 0.0, 1.0);
        CGContextSetStrokeColorWithColor(context, black);
        CFRelease(black);
        
        CGContextBeginPath(context);
        CGContextAddPath(context, path);
        
        CGContextSetLineWidth(context, 3.0);
        
        CGContextStrokePath(context);
        
        CFRelease(path);
        
    }
    
}

@end
